import { Module, MutationTree, ActionTree, GetterTree } from 'vuex';
import { StateInterface } from 'src/store';
import { axios } from 'src/boot/axios';
import { AxiosResponse } from 'axios';

export interface DrinkInterface {
  drinkTypes: FG.DrinkType[];
  drinks: FG.Drink[];
}

const state: DrinkInterface = {
  drinkTypes: [],
  drinks: [],
};

const mutations: MutationTree<DrinkInterface> = {
  setDrinkTypes(state, drinkTypes: FG.DrinkType[]) {
    state.drinkTypes = drinkTypes;
  },
  setDrinkType(state, drinkType: FG.DrinkType) {
    const item = state.drinkTypes.find((item) => item.id == drinkType.id);
    if (item) {
      item.name = drinkType.name;
    }
  },
  addDrinkType(state, drinkType: FG.DrinkType) {
    state.drinkTypes.unshift(drinkType);
  },
  removeDrinkType(state, id: number) {
    const index = state.drinkTypes.findIndex((item) => item.id == id);
    state.drinkTypes.splice(index, 1);
  },
  setDrinks(state, drinks: FG.Drink[]) {
    state.drinks = drinks;
  },
  setDrink(state, drink: FG.Drink) {
    const index = state.drinks.findIndex((item) => (item.id = drink.id));
    if (index) {
      state.drinks[index] = drink;
    } else {
      state.drinks.push(drink);
    }
  },
};

const actions: ActionTree<DrinkInterface, StateInterface> = {
  getDrinkTypes({ commit }) {
    axios
      .get('/pricelist/drink-types')
      .then((response: AxiosResponse<FG.DrinkType[]>) => {
        commit('setDrinkTypes', response.data);
      })
      .catch((err) => console.warn(err));
  },
  addDrinkType({ commit }, data) {
    axios
      .post('/pricelist/drink-types', data)
      .then((response: AxiosResponse<FG.DrinkType>) => {
        commit('addDrinkType', response.data);
      })
      .catch((err) => {
        console.warn(err);
      });
  },
  removeDrinkType({ commit }, data: number) {
    axios
      .delete(`/pricelist/drink-types/${data}`)
      .then(() => {
        commit('removeDrinkType', data);
      })
      .catch((err) => console.warn(err));
  },
  changeDrinkTypeName({ commit }, drinkType: FG.DrinkType) {
    axios
      .put(`/pricelist/drink-types/${drinkType.id}`, drinkType)
      .then(() => {
        commit('setDrinkType', drinkType);
      })
      .catch((err) => console.warn(err));
  },
  getDrinks({ commit }) {
    axios
      .get('/pricelist/drinks')
      .then((response: AxiosResponse<FG.Drink[]>) => {
        commit('setDrinks', response.data);
      })
      .catch((err) => console.warn(err));
  },
  createDrink({ commit }, data) {
    axios
      .post('/pricelist/drinks', data)
      .then((response: AxiosResponse<FG.Drink>) => {
        commit('setDrink', response.data);
      })
      .catch((err) => console.warn(err));
  },
};
const getters: GetterTree<DrinkInterface, StateInterface> = {};

const schedule: Module<DrinkInterface, StateInterface> = {
  namespaced: true,
  state,
  mutations,
  actions,
  getters,
};

export default schedule;
